package org.starobjects.restful.viewer.resources.objects.properties;

import java.text.MessageFormat;

import nu.xom.Element;

import org.nakedobjects.metamodel.adapter.NakedObject;
import org.nakedobjects.metamodel.adapter.oid.Oid;
import org.nakedobjects.metamodel.authentication.AuthenticationSession;
import org.nakedobjects.metamodel.facets.object.parseable.ParseableFacet;
import org.nakedobjects.metamodel.spec.NakedObjectSpecification;
import org.nakedobjects.metamodel.spec.feature.OneToOneAssociation;
import org.starobjects.restful.viewer.html.HtmlClass;
import org.starobjects.restful.viewer.resources.objects.TableColumnNakedObjectAssociation;
import org.starobjects.restful.viewer.xom.ResourceContext;


public final class TableColumnOneToOneAssociationAccess extends TableColumnNakedObjectAssociation<OneToOneAssociation> {

    public TableColumnOneToOneAssociationAccess(
            final AuthenticationSession session,
            final NakedObject nakedObject,
            final ResourceContext resourceContext) {
        super("Access", session, nakedObject, resourceContext);
    }

    @Override
    public Element doTd(final OneToOneAssociation property) {
        if (!property.isVisible(getSession(), getNakedObject()).isAllowed()) {
            return xhtmlRenderer.p(null, HtmlClass.PROPERTY);
        }

        final NakedObject propertyValue = property.get(getNakedObject());
        if (propertyValue == null) {
            return xhtmlRenderer.p(null, HtmlClass.PROPERTY);
        }

        ParseableFacet parseable = property.getFacet(ParseableFacet.class);
        if (parseable == null) {
            final NakedObjectSpecification propertySpec = property.getSpecification();
            parseable = propertySpec.getFacet(ParseableFacet.class);
        }
        if (parseable != null) {
            return doTdForParseable(propertyValue);
        } else {
            return doTdForNonParseable(propertyValue);
        }
    }

    private Element doTdForParseable(final NakedObject propertyValue) {
        final String titleString = propertyValue.titleString();
        return xhtmlRenderer.p(titleString, HtmlClass.PROPERTY);
    }

    private Element doTdForNonParseable(final NakedObject propertyValue) {
        final String titleString = propertyValue.titleString();
        final Oid oid = propertyValue.getOid();
        if (oid == null) {
            return xhtmlRenderer.p(null, HtmlClass.PROPERTY);
        }
        final String uri = MessageFormat.format("{0}/object/{1}", getContextPath(), getOidStr(propertyValue));
        return new Element(xhtmlRenderer.aHref(uri, titleString, "propertyValue", "property", HtmlClass.PROPERTY));
    }

}
